/*------------------------------------------------------------------------
 TcpClient.cpp

 DESCRIPTION
 This program will open a TCP socket, monitor the keyboard (stdin) for
 input, and then send that input out the TCP socket.

 Windows libraries to include:
 ws2_32.lib
 
------------------------------------------------------------------------*/
#include <winsock2.h>
#include <windows.h>
#include <stdio.h>
#include <ctype.h>
#include <io.h>
#include <time.h>

#define RX_BUFSIZE (16000)


// Global vars
bool bExitThread = FALSE;

// Function prototypes
void CloseSocket(SOCKET s);
int ProcessEvents(SOCKET socket);
DWORD WINAPI ReceiveTCP(LPVOID lpvThreadParam);


/*------------------------------------------------------------------------
 main
 ------------------------------------------------------------------------*/
int main (int argc, char ** argv)
{
	SOCKET socketClient;
	char strDestIPAddress[80];
	int	 nDestIPPort;
	int len;
	int result;
	struct sockaddr_in  address;
	bool exit_program = FALSE;
	bool bExitThread  = FALSE;


	printf("TcpClient, Version 0.5, 2/5/2001\n");
	printf("Copyright 2001 NetBurner, Inc.\n");
	printf("All rights reserved.\n\n\n");
	if (argc < 3)
	{
		printf("Usage: TcpClient <dest. address> <dest. port>\n");
		return -1;
	}

	/*
	Initiate use of windows sockets dll for this process.

	The MAKEWORD(<major revision>,<minor revision>) parameter 
	in WSAStartup()	specifies the highest version of Windows
	Sockets support that the caller can use. The high order
	byte specifies the minor version (revision) number; 
	the low-order byte specifies the major version number. 
	*/
	WSADATA wsad;
	WSAStartup(MAKEWORD(1,1), &wsad);


	// create socket
	strncpy(strDestIPAddress, argv[1], 80);
	nDestIPPort = atoi(argv[2]);
	printf("Connecting to %s:%d\n", strDestIPAddress, nDestIPPort);
	socketClient = socket(AF_INET, SOCK_STREAM, 0);
	if (socketClient == INVALID_SOCKET)
	{
		printf("Invalid Socket, Error #%d\n", WSAGetLastError());
		exit(1);
	}

	// name the socket
	address.sin_family = AF_INET;
	address.sin_addr.s_addr = inet_addr(strDestIPAddress);
	address.sin_port = htons(nDestIPPort);
	len = sizeof(address);

	// connect client socket to server socket
	result = connect(socketClient, (struct sockaddr *)&address, len);
	if (result == SOCKET_ERROR) 
	{
		printf("Error opening socket %s:%d, Error #%d\n", 
			    strDestIPAddress, nDestIPPort, WSAGetLastError());
		exit(1);
	}

	if (!exit_program)
	{
		/*
		Create and start thread to receive TCP/IP data.  This function
		has been placed here so that the socket will not be put in
		non-blocking mode for sending files.  This simplifies the
		program with regard to closing the connection because we do 
		not need to determine when the thread has terminated and issue
		a WSAEventSelect() call with networkevents set to 0, followed
		by an ioctlsockt() call to enable blocking.

		HANDLE CreateThread( LPSECURITY_ATTRIBUTES lpThreadAttributes,  // pointer to thread security attributes 
				DWORD dwStackSize,  // initial thread stack size, in bytes 
				LPTHREAD_START_ROUTINE lpStartAddress, 	// pointer to thread function 
				LPVOID lpParameter, 					// argument for new thread 
				DWORD dwCreationFlags,					// creation flags 
				LPDWORD lpThreadId						// pointer to returned thread identifier 
				); 
		*/
		DWORD  dwThreadId;
		HANDLE hThread;
		hThread = CreateThread(NULL, 0, ReceiveTCP, (LPVOID) &socketClient,0, &dwThreadId);

		// Process keyboard input
		char buf[128];
		char *pstrResult;

		printf("Type a string to send (<enter> to exit)\n");
		while (!exit_program)
		{
			pstrResult = fgets(buf, sizeof(buf), stdin);
			if ( (pstrResult == NULL) || (buf[0] == '\n') )
				exit_program = TRUE;

			if (! exit_program)
			{
				int nBytesSent = send(socketClient, buf, strlen(buf), 0);
				printf("Sent %d bytes: %s\n", nBytesSent, buf);
			}
		}
	}
	
	
	printf("Exiting program\n");
	bExitThread = TRUE;
//	CloseSocket(socketClient);
	closesocket(socketClient);
	WSACleanup();
	return 0;
}


/*------------------------------------------------------------------------
 Function to close the network socket
 ------------------------------------------------------------------------*/
void CloseSocket(SOCKET s)
{
	printf("Closing socket...\n");
	DWORD argp = 0;
	ioctlsocket(s, FIONBIO, &argp);		// disable nonblocking
	shutdown(s, SD_SEND);

	char buf[128];
	int nBytesRead;
	do {
		nBytesRead = recv(s, buf, sizeof(buf)-1, 0);
		buf[nBytesRead] = '\0';
	} while ( (nBytesRead != 0) && (nBytesRead != SOCKET_ERROR) );

	if (nBytesRead == SOCKET_ERROR)
		printf("nBytesRead = SOCKET_ERROR\n");

	closesocket(s);
}   



/*------------------------------------------------------------------------
 Receive TCP Thread
 ------------------------------------------------------------------------*/
DWORD WINAPI ReceiveTCP(LPVOID lpvThreadParam)
{
	SOCKET s;

	s = * (SOCKET *)lpvThreadParam;
	ProcessEvents(s);
	return 0;
}



/*------------------------------------------------------------------------
 Function to process stdin and socket events
 ------------------------------------------------------------------------*/
int ProcessEvents(SOCKET socket)
{
	char buf[1024];

	int nBytesRead = recv(socket, buf, sizeof(buf)-1, 0);
	buf[nBytesRead] = '\0';
	printf("Received %d bytes: %s\n", nBytesRead, buf);

	// If this event occurred and there is no data
	// to read, the socket is closed on the other
	// side.
	if (nBytesRead == 0)
		bExitThread = TRUE;

	return 0;
}






